# Copyright (c) 2013 Mirantis Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from sahara import conductor as c


conductor = c.API


def get_raw_data(context, job_binary):
    # url example: 'internal-db://JobBinaryInternal-UUID'
    binary_internal_id = job_binary.url[len("internal-db://"):]
    return conductor.job_binary_internal_get_raw_data(context,
                                                      binary_internal_id)
